/*

 PMMail/2 Incoming REXX Filter - Version 1.8

 HISTORY

    * 1.0 - (11.03.2003) - Erste Verffentlichung
    * 1.1 - (11.03.2003) - ISO-8859-15 zu ISO-8859-1 Konvertierung
    * 1.2 - (15.03.2003) - Zeitzonen-Differenz-Korrektur fr PDT und EDT
    * 1.3 - (25.09.2003) - Erkennung von US-ASCII charsets
    * 1.4 - (14.10.2003) - Reply-To wird ebenfalls auf ISO-Kodierung geprft
    * 1.5 - (   10.2003) - Multipart/Alternative Abschlusemikolon eingefgt
    * 1.6 - (06.11.2003) - ISO-8859-15 zu ISO-8859-1 Konvertierung fr Subject
                           gefixt fr Grobuchstaben
    * 1.7 - (06.03.2004) - charset hinzugefgt, wenn nur Content-Type =
                           text/plain gegeben
    * 1.8 - (06.02.2005) - Korrektur der Kodierung von Multipart Mails
                           (fr den Text-Teil)

 ZWECK DES SKRIPTS

    Dieses Skript behebt einige Probleme mit eingehenden Mails, die nicht
    standardkonform sind, bzw. deren Nichtkonformitt nicht automatisch von
    PMMail korrigiert wird. Insbesondere Mails von eBay oder AOL sind dabei
    auffllig.
    Folgende Probleme werden durch dieses Skript behoben:

    * Kodierung der Mail in ISO-8859-15 oder vllig ohne Angabe, wodurch die
       Mail mit falschen Zeichen dargestellt wird.
    * Subject enthlt Umlaute oder  und ist nicht ISO-8859-1 kodiert,
       wodurch der Betreff mit falschen Zeichen dargestellt wird.
    * From enthlt Umlaute oder  und ist nicht ISO-8859-1 kodiert,
       wodurch der Absender mit falschen Zeichen dargestellt wird.
    * Reply-To enthlt Umlaute oder  und ist nicht ISO-8859-1 kodiert,
       wodurch der Absender mit falschen Zeichen dargestellt wird.
    * Timezone-Angabe aus der Zeitzone PST (PDT) oder EST (EDT) enthlt keine
       Zeitdifferenz, wodurch die Mail mit einem falschen Zeitstempel angezeigt
       bzw. einsortiert wird (Zeit wird als GMT interpretiert).
    * Body enthlt Umlaute oder  und ist nicht ISO-8859-1 kodiert (d.h.
       es ist kein Content-Type, kein Content-Transfer-Encoding und kein
       MIME-Format angegeben).
    * Multipart Nachrichten werden nicht angezeigt, da der Eintrag
       Multipart/Alternative nicht mit einem Semikolon abgeschlossen ist.

    Die fehlenden Eintrge im Header werden ergnzt bzw. die unvollstndigen
    korrigiert. Hierbei kann nicht ausgeschlossen werden, da in einigen
    Mails Eintrge hinzugefgt werden, die dort berhaupt nicht ntig wren.
    Nach bisherigen Erfahrungen, richten berflssige Eintrge aber keinen
    Schaden an.

    Wird das Skript aktiv, so wird dies durch ein akustisches Signal angezeigt.

 EINSATZ DES SKRIPTS

    Um eingehende Mail mit diesem Skript zu bearbeiten, kann es in den
    Account Settings auf der Seite REXX beim Nachrichtenempfang eingetragen
    werden:

    Message Receive Exit -> X:\PMMail\REXX\Incoming.cmd

 HAFTUNGSAUSSCHLUSS

    Ich stelle dieses Skript kostenfrei zur Verfgung und bernehme keinerlei
    Verantwortung fr Schden oder Probleme welcher Art auch immer, die durch
    die Verwendung des Skriptes entstehen sollten. Die Verwendung des Skriptes
    geschieht komplett auf eigene Gefahr.

    Es wre allerdings nett, wenn ich zumindest eine Mail bekme, wenn diese
    Skript bei jemandem zum Einsatz kommen sollte.

 DANK

    Zur Implementation der Prozedur CountStr haben mir die berlegungen von
    Dallas Legan von der Southern California OS/2 User Group (SCOUG) sehr
    geholfen.
     -> http://www.scoug.com/os24u/2002/scoug202.changestr.html

    Fr den grundstzlichen Aufbau des Skripts war mir das Spamcheck Skript
    von Winfried Tilanus sehr hilfreich.
     -> http://www.xs4all.nl/~wtilanus/spam.html

 KONTAKT

    eMail: Axel Hagedorn <axel.hagedorn@gmx.de>

    Ich bin offen fr jeden Verbesserungsvorschlag und natrlich auch jeden
    Hinweis auf mgliche Fehler.

 COPYRIGHT

    2005 - INFINITY/2 - Axel Hagedorn, Darmstadt, Deutschland

*/

CALL RxFuncadd 'SysFileDelete', 'RexxUtil', 'SysFileDelete'

PARSE ARG mailfile

/* Maildatei einlesen */
DO line = 1 BY 1 WHILE LINES(mailfile)
  mail.line = LINEIN(mailfile)
  shadow.line = ''
END

maillength = line - 1 /* Anzahl Zeilen in Maildatei */
newentries = 0        /* Anzahl neuer Zeilen in Mail-Header */

mimever = 0           /* MIME-Type nicht vorhanden */
ctype = 0             /* Content-Type nicht vorhanden */
ctypechar = 0         /* Content-Type enthlt keine charset-Angabe */
ctransenc = 0         /* Content-Transfer-Encoding nicht vorhanden */

boundary = ''

DO line = 1 BY 1 UNTIL LENGTH(mail.line) = 0

  subject = ''
  from = ''
  timezone = ''
  version = ''
  type = ''
  encoding = ''
  version = ''

  /* Header auf Verwendung von ISO-8859-15 berprfen
     Dabei fr Subject etwas genauer prfen, da der gesuchte Text ja auch als "Text" im
     Header stehen knnte */
  IF CountStr('8859-15',mail.line) > 0 THEN
    DO
      IF CountStr('Subject: ',mail.line) = 0 THEN
        DO
          mail.line = ReplStr('8859-15','8859-1',mail.line)
          newentries = newentries + 1
          entry.newentries = 'X-INFINITY/2-Modified: ISO-8859-15 changed to ISO-8859-1'
          CALL BEEP 900,100
        END
      ELSE
        DO
          IF (CountStr('8859-15?q?',mail.line) + CountStr('8859-15?Q?',mail.line)) > 0 THEN
            DO
              mail.line = ReplStr('8859-15?q?','8859-1?q?',mail.line)
              mail.line = ReplStr('8859-15?Q?','8859-1?q?',mail.line)
              newentries = newentries + 1
              entry.newentries = 'X-INFINITY/2-Modified: ISO-8859-15 changed to ISO-8859-1'
              CALL BEEP 900,100
            END
        END
    END

  /* Subject auf ISO Kodierung berprfen */
  PARSE VAR mail.line . 'Subject: ' subject
  IF \(SUBSTR(subject,1,2) = '=?' & SUBSTR(subject,6,7) = '-8859-1') THEN
    DO
      oldsubject = subject
      subject = ReplStr('','=DF',subject)
      subject = ReplStr('','=E4',subject)
      subject = ReplStr('','=F6',subject)
      subject = ReplStr('','=FC',subject)
      subject = ReplStr('','=C4',subject)
      subject = ReplStr('','=D6',subject)
      subject = ReplStr('','=DC',subject)
      IF oldsubject \== subject THEN
        DO
          mail.line = 'Subject: ' || '=?iso-8859-1?q?' || subject || '?='
          newentries = newentries + 1
          entry.newentries = 'X-INFINITY/2-Modified: Subject iso-encoded'
          CALL BEEP 900,100
        END
    END

  /* From auf ISO Kodierung berprfen */
  PARSE VAR mail.line . 'From: ' from
  IF \(SUBSTR(from,1,2) = '=?' & SUBSTR(from,6,7) = '-8859-1') THEN
    DO
      oldfrom = from
      from = ReplStr('','=DF',from)
      from = ReplStr('','=E4',from)
      from = ReplStr('','=F6',from)
      from = ReplStr('','=FC',from)
      from = ReplStr('','=C4',from)
      from = ReplStr('','=D6',from)
      from = ReplStr('','=DC',from)
      IF oldfrom \== from THEN
        DO
          mail.line = 'From: ' || '=?iso-8859-1?q?' || from || '?='
          newentries = newentries + 1
          entry.newentries = 'X-INFINITY/2-Modified: From iso-encoded'
          CALL BEEP 900,100
        END
    END

  /* Reply-To auf ISO Kodierung berprfen */
  PARSE VAR mail.line . 'Reply-To: ' replyto
  IF \(SUBSTR(replyto,1,2) = '=?' & SUBSTR(replyto,6,7) = '-8859-1') THEN
    DO
      oldreplyto = replyto
      replyto = ReplStr('','=DF',replyto)
      replyto = ReplStr('','=E4',replyto)
      replyto = ReplStr('','=F6',replyto)
      replyto = ReplStr('','=FC',replyto)
      replyto = ReplStr('','=C4',replyto)
      replyto = ReplStr('','=D6',replyto)
      replyto = ReplStr('','=DC',replyto)
      IF oldreplyto \== replyto THEN
        DO
          mail.line = 'Reply-To: ' || '=?iso-8859-1?q?' || replyto || '?='
          newentries = newentries + 1
          entry.newentries = 'X-INFINITY/2-Modified: Reply-To iso-encoded'
          CALL BEEP 900,100
        END
    END

  /* PST/PDT-Zeitzone (ebay) auf Vollstndigkeit prfen
     EST/EDT-Zeitzone (aol) auf Vollstndigkeit prfen */
  PARSE VAR mail.line . 'Date: ' part1 ':' part2 ':' part3 ' ' timezone
  IF (CountStr('PST',timezone) > 0 & CountStr('0',timezone) = 0) THEN
    DO
      mail.line = 'Date: ' || part1 || ':' || part2 || ':' || part3 || ' -0800 (PST)'
      newentries = newentries + 1
      entry.newentries = 'X-INFINITY/2-Modified: Date fully qualified'
      CALL BEEP 900,100
    END
  IF (CountStr('PDT',timezone) > 0 & CountStr('0',timezone) = 0) THEN
    DO
      mail.line = 'Date: ' || part1 || ':' || part2 || ':' || part3 || ' -0700 (PDT)'
      newentries = newentries + 1
      entry.newentries = 'X-INFINITY/2-Modified: Date fully qualified'
      CALL BEEP 900,100
    END
  IF (CountStr('EST',timezone) > 0 & CountStr('0',timezone) = 0) THEN
    DO
      mail.line = 'Date: ' || part1 || ':' || part2 || ':' || part3 || ' -0500 (EST)'
      newentries = newentries + 1
      entry.newentries = 'X-INFINITY/2-Modified: Date fully qualified'
      CALL BEEP 900,100
    END
  IF (CountStr('EDT',timezone) > 0 & CountStr('0',timezone) = 0) THEN
    DO
      mail.line = 'Date: ' || part1 || ':' || part2 || ':' || part3 || ' -0400 (EDT)'
      newentries = newentries + 1
      entry.newentries = 'X-INFINITY/2-Modified: Date fully qualified'
      CALL BEEP 900,100
    END

  /* MIME-Version auf Existenz prfen */
  PARSE UPPER VAR mail.line . 'MIME-VERSION: ' version
  IF LENGTH(version) > 0 THEN mimever = 1

  /* Content-Type auf Existenz berprfen und wenn ja, ob es einen charset beschreibt.
     Wenn nicht, und Content-type ist text/plain, prfen, ob Content-type ber mehrere Zeilen geht -
     wenn nicht, dann charset iso-8859-1 anhngen.
     Weiterhin prfen, ob Multipart/Alternative mit einem Semikolon abgeschlossen ist.
     Wenn Content-Type Multipart ist, mu die Boundary ausgelesen werden, um anschlieend den Body zu scannen. */
  PARSE UPPER VAR mail.line . 'CONTENT-TYPE: ' type
  IF type \= '' THEN
    DO
      ctype = 1
      IF CountStr('CHARSET=',type) > 0 THEN ctypechar = 1
        ELSE
          IF CountStr('TEXT/PLAIN;',type) > 0 THEN
            DO
              lineone = line + 1
              PARSE UPPER VAR mail.lineone part1 'CHARSET' part2
              linetwo = lineone + 1
              PARSE UPPER VAR mail.linetwo part1 'CHARSET' part3
              IF (part2 = '') & (part3 = '') THEN
                DO
                  mail.line = 'Content-Type: text/plain; charset="iso-8859-1"'
                  newentries = newentries + 1
                  entry.newentries = 'X-INFINITY/2-Modified: charset="iso-8859-1" added to Content-Type'
                  CALL BEEP 900,100
                END
            END
      IF CountStr('US-ASCII',type) > 0 THEN ctypechar = 2
      IF (CountStr('MULTIPART/ALTERNATIVE',type) > 0 & CountStr('MULTIPART/ALTERNATIVE;',type) = 0)
        THEN
        DO
          PARSE VAR mail.line part1 ': ' part2 ' ' part3
          mail.line = 'Content-Type: ' || 'Multipart/Alternative; ' || part3
          newentries = newentries + 1
          entry.newentries = 'X-INFINITY/2-Modified: Multipart/Alternative trailing ; added'
          CALL BEEP 900,100
          CALL BEEP 900,100
        END
      IF (CountStr('MULTIPART',type) > 0)
        THEN
        DO
          IF (CountStr('BOUNDARY',type) > 0) THEN
            PARSE VAR mail.line part1 '="' boundary '"' part3
            ELSE
              DO
                lineone = line + 1
                PARSE VAR mail.lineone part1 '="' boundary '"' part3
              END
        END
    END

  /* Content-Transfer-Encoding auf Existenz berprfen */
  PARSE UPPER VAR mail.line . 'CONTENT-TRANSFER-ENCODING: ' encoding
  IF LENGTH(encoding) > 0 THEN ctransenc = 1

END

/* MIME-Version nicht vorhanden, MIME-Version: 1.0 wird angenommen */
IF (mimever = 0) THEN
  DO
    newentries = newentries + 1
    entry.newentries = 'MIME-Version: 1.0'
    newentries = newentries + 1
    entry.newentries = 'X-INFINITY/2-Added: MIME-Version: 1.0'
    CALL BEEP 900,100
  END

/* Content-Type nicht vorhanden, Content-Type: text/plain;
   charset="iso-8859-1" wird angenommen */
IF ctype = 0 THEN
  DO
    newentries = newentries + 1
    entry.newentries = 'Content-Type: text/plain; charset="iso-8859-1"'
    newentries = newentries + 1
    entry.newentries = 'X-INFINITY/2-Added: Content-Type: text/plain; charset="iso-8859-1"'
    CALL BEEP 900,100
    ctypechar = 1
  END

/* Content-Transfer-Encoding nicht vorhanden,
   Content-Transfer-Encoding: 8bit wird angenommen, wenn kein US-ASCII */
IF (ctypechar > 0 & ctransenc = 0) THEN
  DO
    newentries = newentries + 1
    IF (ctypechar = 1) THEN entry.newentries = 'Content-Transfer-Encoding: 8bit'
      ELSE entry.newentries = 'Content-Transfer-Encoding: 7bit'
    newentries = newentries + 1
    IF (ctypechar = 1) THEN entry.newentries = 'X-INFINITY/2-Added: Content-Transfer-Encoding: 8bit'
      ELSE entry.newentries = 'X-INFINITY/2-Added: Content-Transfer-Encoding: 7bit'
    CALL BEEP 900,100
  END

headerlength = line /* Anzahl Zeilen in Mail-Header */

/* Boundary wurde ausgelesen - es mu geprft werden, ob der Text-Part eine korrekte Codierung enthlt.
   D.h. der Body mu gescannt werden. */
IF (boundary > '') THEN
  DO
    boundary = '--' || boundary
    DO line = headerlength TO maillength
      IF mail.line = boundary THEN
         DO
          lineone = line + 1
          PARSE UPPER VAR mail.lineone part1 'TEXT/PLAIN' part2
          IF part2 > 0 THEN
            DO
              /*gar kein charset?*/
              PARSE UPPER VAR mail.lineone part1 'CHARSET' part2
              linetwo = lineone + 1
              PARSE UPPER VAR mail.linetwo part1 'CHARSET' part3
              IF (part2 = '') & (part3 = '') THEN
                DO
                  mail.lineone = 'Content-Type: text/plain; charset="iso-8859-1"'
                  newentries = newentries + 1
                  entry.newentries = 'X-INFINITY/2-Modified: charset="iso-8859-1" added to Content-Type of text/plain mailpart'
                  CALL BEEP 900,100
                  CALL BEEP 900,100
                END
                ELSE
                  IF CountStr('8859-15',part2) > 0 THEN
                    DO
                      mail.lineone = ReplStr('8859-15','8859-1',mail.lineone)
                      newentries = newentries + 1
                      entry.newentries = 'X-INFINITY/2-Modified: ISO-8859-15 changed to ISO-8859-1 in text/plain mailpart'
                      CALL BEEP 900,100
                      CALL BEEP 900,100
                    END
                    ELSE
                      IF CountStr('8859-15',part3) > 0 THEN
                        DO
                          mail.linetwo = ReplStr('8859-15','8859-1',mail.linetwo)
                          newentries = newentries + 1
                          entry.newentries = 'X-INFINITY/2-Modified: ISO-8859-15 changed to ISO-8859-1 in text/plain mailpart'
                          CALL BEEP 900,100
                          CALL BEEP 900,100
                        END
              PARSE UPPER VAR mail.lineone . 'CONTENT-TRANSFER-ENCODING: ' encoding
              IF LENGTH(encoding) = 0 THEN
                DO
                  PARSE UPPER VAR mail.linetwo . 'CONTENT-TRANSFER-ENCODING: ' encoding
                  IF LENGTH(encoding) = 0 THEN
                    DO
                      linethree = linetwo + 1
                      PARSE UPPER VAR mail.linethree . 'CONTENT-TRANSFER-ENCODING: ' encoding
                      IF LENGTH(encoding) = 0 THEN
                        DO
                          shadow.line = 'Content-Transfer-Encoding: 8bit'
                          newentries = newentries + 1
                          entry.newentries = 'X-INFINITY/2-Added: Content-Transfer-Encoding: 8bit to text/plain mailpart'
                          CALL BEEP 900,100
                          CALL BEEP 900,100
                        END
                    END
                END
            END
        END
    END
  END

/* Maildatei schlieen */
rc = STREAM(mailfile, 'c', 'close')

/* Maildatei neu schreiben falls gendert */
IF newentries > 0 THEN
  DO
    rc = SysFileDelete(mailfile)
    DO line = 1 TO maillength
      IF line = headerlength THEN
        DO newline = 1 to newentries
          rc = lineout(mailfile, entry.newline)
        END
      rc = lineout(mailfile, mail.line)
      IF shadow.line > '' THEN rc = lineout(mailfile, shadow.line)
    END
  END

EXIT


/* Prozedur CountStr - Zum Zhlen der Hufigkeit von Substrings */
CountStr: PROCEDURE

PARSE ARG substring, string

IF substring = '' THEN RETURN 0

precount = 0

substringlength = Length(substring)
PARSE VAR string . '' -(substringlength) lastsubstring
IF lastsubstring == substring THEN precount = 1

DO count = 0 BY 1 UNTIL string == ''
  PARSE VAR string . (substring) string
END

count = count + precount

RETURN count


/* Prozedur ReplStr - Zum Ersetzen von Substrings */
ReplStr: PROCEDURE

PARSE ARG oldsubstring, newsubstring, string

IF oldsubstring = '' THEN RETURN 0

DO i = 1 TO CountStr(oldsubstring, string)
  PARSE VAR string firstpart (oldsubstring) lastpart
  string = firstpart || newsubstring || lastpart
END

RETURN string
